"""

  Application class.

  @author Juan Pedro Bolivar Puente
  @date   April 2011

"""

import pygtk
pygtk.require ('2.0')
import gtk

import util
import doc

GUI_MAIN = 'gui/main.glade'
GUI_ABOUT = 'gui/about.glade'

class GfitsApplication (object):

    _document = None
    _curr_hdu = None
    
    def __init__ (self, *a, **k):
        super (GfitsApplication, self).__init__ (*a, **k)

        builder = gtk.Builder ()
        builder.add_from_file (GUI_MAIN)
        builder.connect_signals (self)

        window      = builder.get_object ('main-window')
        notebook    = builder.get_object ('main-notebook')
        box_toolbar = builder.get_object ('box-toolbar')

        window.show ()

        self._box_toolbar = box_toolbar
        self._window      = window
        self._notebook    = notebook 
        
    def run (self):
        gtk.main ()

    def handle_open (self, win):
        with util.destroying (gtk.FileChooserDialog (
            title   = None,
            action  = gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                       gtk.STOCK_OPEN,   gtk.RESPONSE_OK))) \
        as dialog:    
            filt = gtk.FileFilter ()
            filt.set_name ('FITS image')
            filt.add_mime_type ('image/fits')
            filt.add_pattern ("*.fits")
            dialog.add_filter (filt)
            
            response = dialog.run ()
            if response == gtk.RESPONSE_OK:
                self.open_fits (dialog.get_filename ())

    def open_fits (self, filename):
        if self._document:
            self.close_fits ()

        try:
            self._document = doc.FitsDocument (filename)
            for x in self._document.hdus:
                self._notebook.append_page (x.page, x.label)

            if self._notebook.get_n_pages () > 0:
                self._notebook.set_current_page (0)

        except doc.FitsError, err:
            util.show_error_dialog (
                'Error while loading FITS file',
                'Error while loading FITS file: ' + filename +
                '.\n' + str (err))

    def close_fits (self):
        self._document.close ()
        while self._notebook.get_n_pages ():
            self._notebook.remove_page (-1)
        
    def handle_close (self, obj):
        self._window.destroy ()

    def handle_show_about (self, obj):
        builder = gtk.Builder ()
        builder.add_from_file (GUI_ABOUT)
        about   = builder.get_object ('about-dialog')
        about.run ()
        about.destroy ()
        
    def handle_destroy (self, win):
        gtk.main_quit ()

    def handle_change_page (self, obj, page, page_num):
        if self._document is None:
            return
        
        if self._curr_hdu is not None and \
               self._curr_hdu.toolbar is not None:
            self._box_toolbar.remove (self._curr_hdu.toolbar)

        self._curr_hdu = self._document.hdus [page_num]
        if self._curr_hdu.toolbar:
            self._box_toolbar.add (self._curr_hdu.toolbar)
        
